#ifndef PYTHONIC_IO__IO_TEXTIOWRAPPER_READLINE_HPP
#define PYTHONIC_IO__IO_TEXTIOWRAPPER_READLINE_HPP

#include "pythonic/include/io/_io/TextIOWrapper/readline.hpp"
#include "pythonic/builtins/file/readline.hpp"

#endif
