

set -ex



pip check
pythran --version
pythran --help
pythran-config -vvv
pythran -v dprod.py
python -c "import dprod"
pythran -v simple_numexpr.py -DUSE_XSIMD -march=native
python -c "import simple_numexpr"
exit 0
