#ifndef PYTHONIC_INCLUDE_OPERATOR_IAND__HPP
#define PYTHONIC_INCLUDE_OPERATOR_IAND__HPP

#include "pythonic/include/operator_/iand.hpp"

PYTHONIC_NS_BEGIN

namespace operator_
{

  USING_FUNCTOR(__iand__, iand);
}
PYTHONIC_NS_END

#endif
