#ifndef PYTHONIC_NUMPY_RANDOM_RANDOM_SAMPLE_HPP
#define PYTHONIC_NUMPY_RANDOM_RANDOM_SAMPLE_HPP

#include "pythonic/include/numpy/random/random_sample.hpp"
#include "pythonic/numpy/random/random.hpp"

#endif
