#ifndef PYTHONIC_INCLUDE_OPERATOR_IXOR_HPP
#define PYTHONIC_INCLUDE_OPERATOR_IXOR_HPP

#define OPERATOR_NAME ixor
#define OPERATOR_SYMBOL ^
#define OPERATOR_ISYMBOL ^=
#include "pythonic/include/operator_/icommon.hpp"

#endif
