#ifndef PYTHONIC_OPERATOR_IDIV_HPP
#define PYTHONIC_OPERATOR_IDIV_HPP

#include "pythonic/include/operator_/idiv.hpp"

#define OPERATOR_NAME idiv
#define OPERATOR_SYMBOL /
#define OPERATOR_ISYMBOL /=

#include "pythonic/operator_/icommon.hpp"

#endif
