#ifndef PYTHONIC_INCLUDE_OPERATOR_ICONCAT__HPP
#define PYTHONIC_INCLUDE_OPERATOR_ICONCAT__HPP

#include "pythonic/include/operator_/iconcat.hpp"

PYTHONIC_NS_BEGIN

namespace operator_
{

  USING_FUNCTOR(__iconcat__, iconcat);
}
PYTHONIC_NS_END

#endif
